//
//  VotlageInputController.m
//  VoltageInput
//
//  Created by Phidgets on 2015-07-21.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "VotlageInputController.h"

@implementation VotlageInputController

typedef struct {
    const char *name;
    PhidgetVoltageInput_SensorType type;
} VoltageInputSensor;

static const VoltageInputSensor sensorList[] ={
    {"Generic Voltage Sensor",                                                          SENSOR_TYPE_VOLTAGE},
    {"1114 - Temperature Sensor",                                                       SENSOR_TYPE_1114},
    {"1117 - Voltage Sensor",                                                           SENSOR_TYPE_1117},
    {"1123 - Precision Voltage Sensor",                                                 SENSOR_TYPE_1123},
    {"1127 - Precision Light Sensor",                                                   SENSOR_TYPE_1127},
    {"1130 - pH Adapter",                                                               SENSOR_TYPE_1130_PH},
    {"1130 - ORP Adapter",                                                              SENSOR_TYPE_1130_ORP},
    {"1132 - 4-20mA Adapter",                                                           SENSOR_TYPE_1132},
    {"1133 - Sound Sensor",                                                             SENSOR_TYPE_1133},
    {"1135 - Precision Voltage Sensor",                                                 SENSOR_TYPE_1135},
    {"1142 - Light Sensor 1000 lux",                                                    SENSOR_TYPE_1142},
    {"1143 - Light Sensor 70000 lux",                                                   SENSOR_TYPE_1143},
    {"3500 - AC Current Sensor 10Amp lux",                                              SENSOR_TYPE_3500},
    {"3501 - AC Current Sensor 25Amp",                                                  SENSOR_TYPE_3501},
    {"3502 - AC Current Sensor 50Amp",                                                  SENSOR_TYPE_3502},
    {"3503 - AC Current Sensor 100Amp",                                                 SENSOR_TYPE_3503},
    {"3507 - AC Voltage Sensor 0-250V (50 Hz)",                                         SENSOR_TYPE_3507},
    {"3508 - AC Voltage Sensor 0-250V (60 Hz)",                                         SENSOR_TYPE_3508},
    {"3509 - DC Voltage Sensor 0-200V",                                                 SENSOR_TYPE_3509},
    {"3510 - DC Voltage Sensor 0-75V",                                                  SENSOR_TYPE_3510},
    {"3511 - DC Current Sensor 0-10mA",                                                 SENSOR_TYPE_3511},
    {"3512 - DC Current Sensor 0-100mA",                                                SENSOR_TYPE_3512},
    {"3513 - DC Current Sensor 0-1A",                                                   SENSOR_TYPE_3513},
    {"3514 - AC Active Power Sensor 0-250V*0-30A (50Hz)",                               SENSOR_TYPE_3514},
    {"3515 - AC Active Power Sensor 0-250V*0-30A (60Hz)",                               SENSOR_TYPE_3515},
    {"3516 - AC Active Power Sensor 0-250V*0-5A (50Hz)",                                SENSOR_TYPE_3516},
    {"3517 - AC Active Power Sensor 0-250V*0-5A (60Hz)",                                SENSOR_TYPE_3517},
    {"3518 - AC Active Power Sensor 0-110V*0-5A (60Hz)",                                SENSOR_TYPE_3518},
    {"3519 - AC Active Power Sensor 0-110V*0-15A (60Hz)",                               SENSOR_TYPE_3519},
    {"3584 - 0-50A DC Current Transducer",                                              SENSOR_TYPE_3584},
    {"3585 - 0-100A DC Current Transducer",                                             SENSOR_TYPE_3585},
    {"3586 - 0-250A DC Current Transducer",                                             SENSOR_TYPE_3586},
    {"3587 - +-50A DC Current Transducer",                                              SENSOR_TYPE_3587},
    {"3588 - +-100A DC Current Transducer",                                             SENSOR_TYPE_3588},
    {"3589 - +-250A DC Current Transducer",                                             SENSOR_TYPE_3589},
    {NULL, 0}
};

static PhidgetVoltageInput_SensorType getSensorType(const char *str) {
    static const VoltageInputSensor *sensor;
    
    for (sensor = sensorList; sensor->name != NULL; sensor++)
        if (!strcmp(str, sensor->name))
            return (sensor->type);
    
    return (SENSOR_TYPE_VOLTAGE);
}

- (void)windowWillClose:(NSNotification*)aNotification {
    if(ch)
    {
        //Safely close example
        Phidget_setOnAttachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnDetachHandler((PhidgetHandle)ch, NULL, NULL);
        Phidget_setOnErrorHandler((PhidgetHandle)ch, NULL, NULL);
        PhidgetVoltageInput_setOnVoltageChangeHandler(ch, NULL, NULL);
        Phidget_close((PhidgetHandle)ch);
        PhidgetVoltageInput_delete(&ch);
        ch = NULL;
    }
}


-(void)awakeFromNib{
    PhidgetReturnCode result;
    static const VoltageInputSensor *sensor;
    
    //Manage GUI
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
    [mainWindow setDelegate:self];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    [phidgetInfoBox setHidden:NO];
    
    for (sensor = sensorList; sensor->name != NULL; sensor++)
        [sensorTypeCombo addItemWithTitle:[NSString stringWithCString:sensor->name encoding:NSASCIIStringEncoding]];
    
    result = PhidgetVoltageInput_create(&ch);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = [self initChannel:(PhidgetHandle)ch];
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = PhidgetVoltageInput_setOnVoltageChangeHandler(ch, gotVoltageChangeData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
	
    result = PhidgetVoltageInput_setOnSensorChangeHandler(ch, gotSensorChangeData, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    [PhidgetInfoBox openCmdLine:(PhidgetHandle)ch];
}

-(PhidgetReturnCode)initChannel:(PhidgetHandle) channel{
    PhidgetReturnCode result;
    
    result = Phidget_setOnAttachHandler(channel, gotAttach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnDetachHandler(channel, gotDetach, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    result = Phidget_setOnErrorHandler(channel, gotError, (__bridge void*)self);
    if(result != EPHIDGET_OK){
        return result;
    }
    
    /*
     * Please review the Phidget22 channel matching documentation for details on the device
     * and class architecture of Phidget22, and how channels are matched to device features.
     */
    
    /*
     * Specifies the serial number of the device to attach to.
     * For VINT devices, this is the hub serial number.
     *
     * The default is any device.
     */
    // Phidget_setDeviceSerialNumber(ch, <YOUR DEVICE SERIAL NUMBER>);
    
    /*
     * For VINT devices, this specifies the port the VINT device must be plugged into.
     *
     * The default is any port.
     */
    // Phidget_setHubPort(ch, 0);
    
    /*
     * Specifies that the channel should only match a VINT hub port.
     * The only valid channel id is 0.
     *
     * The default is 0 (false), meaning VINT hub ports will never match
     */
    // Phidget_setIsHubPortDevice(ch, 1);
    
    /*
     * Specifies which channel to attach to.  It is important that the channel of
     * the device is the same class as the channel that is being opened.
     *
     * The default is any channel.
     */
    // Phidget_setChannel(ch, 0);
    
    /*
     * In order to attach to a network Phidget, the program must connect to a Phidget22 Network Server.
     * In a normal environment this can be done automatically by enabling server discovery, which
     * will cause the client to discovery and connect to available servers.
     *
     * To force the channel to only match a network Phidget, set remote to 1.
     */
    // PhidgetNet_enableServerDiscovery(PHIDGETSERVER_DEVICEREMOTE);
    // Phidget_setIsRemote(ch, 1);
    
    return EPHIDGET_OK;
}


#pragma mark Event callbacks
static void gotAttach(PhidgetHandle phid, void *context) {
    [(__bridge id)context performSelectorOnMainThread:@selector(onAttachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotDetach(PhidgetHandle phid, void *context) {
    [(__bridge id)context performSelectorOnMainThread:@selector(onDetachHandler)
                                           withObject:nil
                                        waitUntilDone:NO];
}
static void gotError(PhidgetHandle phid, void *context, Phidget_ErrorEventCode errcode, const char *error) {
    [(__bridge id)context performSelectorOnMainThread:@selector(errorHandler:)
                                           withObject:[NSArray arrayWithObjects:[NSNumber numberWithInt:errcode], [NSString stringWithUTF8String:error], nil]
                                        waitUntilDone:NO];
}
static void gotVoltageChangeData(PhidgetVoltageInputHandle phid, void *context, double voltage) {
        [(__bridge id)context performSelectorOnMainThread:@selector(onVoltageChangeHandler:)
                                               withObject:[NSNumber numberWithDouble:voltage]
                                            waitUntilDone:NO];
}
static void gotSensorChangeData(PhidgetVoltageInputHandle phid, void *context, double sensorVale, Phidget_UnitInfo *sensorUnit) {
        [(__bridge id)context performSelectorOnMainThread:@selector(onSensorChangeHandler:)
                                               withObject:[NSNumber numberWithDouble:sensorVale]
                                            waitUntilDone:NO];
}


Phidget_ChannelSubclass subClass;
#pragma mark Attach, detach, data, and error events
- (void)onAttachHandler{
    uint32 minDataInterval, maxDataInterval, dataInterval;
    int heightChange = 0;
    double minVoltageChangeTrigger,maxVoltageChangeTrigger, voltageChangeTrigger;
    
    //Get information from channel that will allow us to configure the GUI properly
    Phidget_getDeviceID((PhidgetHandle)ch, &deviceID);
    PhidgetVoltageInput_getMinDataInterval(ch, &minDataInterval);
    PhidgetVoltageInput_getMaxDataInterval(ch, &maxDataInterval);
    PhidgetVoltageInput_getDataInterval(ch, &dataInterval);
    PhidgetVoltageInput_getMinVoltageChangeTrigger(ch, &minVoltageChangeTrigger);
    PhidgetVoltageInput_getMaxVoltageChangeTrigger(ch, &maxVoltageChangeTrigger);
    PhidgetVoltageInput_getVoltageChangeTrigger(ch, &voltageChangeTrigger);
    Phidget_getChannelSubclass((PhidgetHandle)ch, &subClass);
    
    //Adjust GUI based on information from channel
    [self hideAll]; //clear everything to ensure proper GUI state
    [phidgetInfoBoxView fillPhidgetInfo:(PhidgetHandle)ch];
    [dataBox setHidden:NO];
    [dataIntervalBox setHidden:NO];

    //Configure the data interval slider with information read from channel
    [minDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",minDataInterval]];
    [maxDataIntervalLabel setStringValue:[NSString stringWithFormat:@"%d ms",maxDataInterval]];
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", dataInterval]];
    [dataIntervalSlider setMinValue:(double)minDataInterval];
    [dataIntervalSlider setMaxValue:(double)maxDataInterval];
    [dataIntervalSlider setIntValue:dataInterval];
    [dataIntervalBox setHidden:NO];
    
    
    //Check if device has configurable change trigger
    if(minVoltageChangeTrigger == maxVoltageChangeTrigger){
        [changeTriggerBox setHidden:YES];
    }
    else{
        [changeTriggerBox setHidden:NO];
        [maxChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f V",maxVoltageChangeTrigger]];
        [minChangeTriggerLabel setStringValue:[NSString stringWithFormat:@"%0.3f V",minVoltageChangeTrigger]];
        [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f V", voltageChangeTrigger]];
        [changeTriggerSlider setMaxValue:maxVoltageChangeTrigger];
        [changeTriggerSlider setMinValue:minVoltageChangeTrigger];
        [changeTriggerSlider setIntValue:voltageChangeTrigger];
    }
    
    switch(deviceID){
        case PHIDID_ADP1000:{
            PhidgetVoltageInput_VoltageRange voltageRange;
            [voltageRangeBox setHidden:NO];
            [voltageRangeCombo insertItemWithTitle:@"400 mV" atIndex:0];
            [voltageRangeCombo insertItemWithTitle:@"2 V" atIndex:1];
            PhidgetVoltageInput_getVoltageRange(ch, &voltageRange);
            if(voltageRange == VOLTAGE_RANGE_400mV)
                [voltageRangeCombo selectItemAtIndex:0];
            else if(voltageRange == VOLTAGE_RANGE_2V)
                [voltageRangeCombo selectItemAtIndex:1];
        }
            break;
        case PHIDID_DAQ1400:
            [powerSupplyBox setHidden:NO];
            break;
        case PHIDID_VCP1000:{
            PhidgetVoltageInput_VoltageRange voltageRange;
            [voltageRangeBox setHidden:NO];
            [voltageRangeCombo insertItemWithTitle:@"312.5 mV" atIndex:0];
            [voltageRangeCombo insertItemWithTitle:@"40 V" atIndex:1];
            PhidgetVoltageInput_getVoltageRange(ch, &voltageRange);
            if(voltageRange == VOLTAGE_RANGE_312_5mV)
                [voltageRangeCombo selectItemAtIndex:0];
            else
                [voltageRangeCombo selectItemAtIndex:1];
        }
            break;
        case PHIDID_VCP1001:{
            PhidgetVoltageInput_VoltageRange voltageRange;
            [voltageRangeBox setHidden:NO];
            [voltageRangeCombo insertItemWithTitle:@"5 V" atIndex:0];
            [voltageRangeCombo insertItemWithTitle:@"15 V" atIndex:1];
            [voltageRangeCombo insertItemWithTitle:@"40 V" atIndex:2];
            [voltageRangeCombo insertItemWithTitle:@"Automatic" atIndex:3];
            PhidgetVoltageInput_getVoltageRange(ch, &voltageRange);
            if(voltageRange == VOLTAGE_RANGE_5V)
                [voltageRangeCombo selectItemAtIndex:0];
            else if(voltageRange == VOLTAGE_RANGE_15V)
                [voltageRangeCombo selectItemAtIndex:1];
            else if(voltageRange == VOLTAGE_RANGE_40V)
                [voltageRangeCombo selectItemAtIndex:2];
            else if(voltageRange == VOLTAGE_RANGE_AUTO)
                [voltageRangeCombo selectItemAtIndex:3];
        }
            break;
        case PHIDID_VCP1002:{
            PhidgetVoltageInput_VoltageRange voltageRange;
            [voltageRangeBox setHidden:NO];
            [voltageRangeCombo insertItemWithTitle:@"10 mV" atIndex:0];
            [voltageRangeCombo insertItemWithTitle:@"40 mV" atIndex:1];
            [voltageRangeCombo insertItemWithTitle:@"200 mV" atIndex:2];
            [voltageRangeCombo insertItemWithTitle:@"1000 mV" atIndex:3];
            [voltageRangeCombo insertItemWithTitle:@"Automatic" atIndex:4];
            PhidgetVoltageInput_getVoltageRange(ch, &voltageRange);
            if(voltageRange == VOLTAGE_RANGE_10mV)
                [voltageRangeCombo selectItemAtIndex:0];
            else if(voltageRange == VOLTAGE_RANGE_40mV)
                [voltageRangeCombo selectItemAtIndex:1];
            else if(voltageRange == VOLTAGE_RANGE_200mV)
                [voltageRangeCombo selectItemAtIndex:2];
            else if(voltageRange == VOLTAGE_RANGE_1000mV)
                [voltageRangeCombo selectItemAtIndex:3];
            else if(voltageRange == VOLTAGE_RANGE_AUTO)
                [voltageRangeCombo selectItemAtIndex:4];
        }
            break;
        case PHIDID_VOLTAGEINPUT_PORT:
        case PHIDID_DAQ1000:
        case PHIDID_1011:
        case PHIDID_1010_1013_1018_1019:
        case PHIDID_1202_1203:
        case PHIDID_1065:{
            [sensorBox setHidden:NO];
        }
            break;
        default:
            break;

    }

    //Adjusting window height
    int rightSideHeight = 0, leftSideHeight = 0;
    if(![sensorBox isHidden])
        rightSideHeight += dataBox.frame.size.height + sensorBox.frame.size.height;
    else{
        leftSideHeight = 15; //acount for settings label
        if(![changeTriggerBox isHidden])
            leftSideHeight += changeTriggerBox.frame.size.height;
        if(![voltageRangeBox isHidden])
            leftSideHeight += voltageRangeBox.frame.size.height;
        if(![dataIntervalBox isHidden])
            leftSideHeight += dataIntervalBox.frame.size.height;
        if(![powerSupplyBox isHidden])
            leftSideHeight += powerSupplyBox.frame.size.height;
    }
    heightChange = (leftSideHeight > rightSideHeight) ? leftSideHeight : rightSideHeight;
    NSRect frame = [mainWindow frame];
    frame.size.width = 550;
    frame.size.height += heightChange;
    frame.origin.y -= heightChange;
    [mainWindow setFrame:frame display:NO animate:NO];
    [mainWindow center];
}

- (void)onDetachHandler{
    //Reset
    lastBox = nil;
    [voltageText setStringValue:@""];
    [sensorTypeCombo removeAllItems];
    [voltageRangeCombo removeAllItems];
    
    //Taking items out of view
    [self hideAll];
    [phidgetInfoBoxView fillPhidgetInfo:nil];
    
    //Adjusting window height
    NSRect frame = [mainWindow frame];
    frame.size.height = [mainWindow minSize].height;
    [mainWindow setFrame:frame display:NO];
    [mainWindow center];
}

-(void) hideAll{
    [dataIntervalBox setHidden:YES];
    [changeTriggerBox setHidden:YES];
    [voltageRangeBox setHidden:YES];
    [dataBox setHidden:YES];
    [sensorBox setHidden:YES];
    [powerSupplyBox setHidden:YES];
}

static int errorCounter = 0;
-(void) outputError:(const char *)errorString{
    errorCounter++;
    NSAttributedString *outputString = [[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@\n",[NSString stringWithUTF8String:errorString]]
    	attributes:@{NSForegroundColorAttributeName: NSColor.controlTextColor}];
    [[errorEventLog textStorage] beginEditing];
    [[errorEventLog textStorage] appendAttributedString:outputString];
    [[errorEventLog textStorage] endEditing];
    
    [errorEventLogCounter setIntValue:errorCounter];
    if(![errorEventLogWindow isVisible])
        [errorEventLogWindow setIsVisible:YES];
}

-(void)errorHandler:(NSArray *)errorEventData{
    const char* errorString = [[errorEventData objectAtIndex:1] UTF8String];
    [self outputError:errorString];
}

-(void)outputLastError {
	const char *errorString;
	char *errorDetailString = NULL;
    size_t errorDetailStringLen = 0;
    PhidgetReturnCode lastErr;
	
    if (!Phidget_getLastError(&lastErr, &errorString, NULL, &errorDetailStringLen)) {
		errorDetailString = malloc(errorDetailStringLen);
		Phidget_getLastError(&lastErr, &errorString, errorDetailString, &errorDetailStringLen);
    	[self outputError:errorDetailString];
    	free(errorDetailString);
    }
}

- (IBAction)clearErrorLog:(id)sender{
    [[errorEventLog textStorage] setAttributedString:[[NSAttributedString alloc] initWithString:@""]];
    [errorEventLogCounter setIntValue:0];
    errorCounter = 0;
}

- (void)onVoltageChangeHandler:(NSNumber *)voltage{

    [voltageText setStringValue:[NSString stringWithFormat:@"%0.4f V", [voltage doubleValue]]];
}

- (void)onSensorChangeHandler:(NSNumber *)sensorVal{
    Phidget_UnitInfo unit;
	
	PhidgetVoltageInput_getSensorUnit(ch, &unit);
	[sensorText setStringValue:[[NSString stringWithFormat:@"%0.3f ",[sensorVal doubleValue]] stringByAppendingString:[NSString stringWithUTF8String:unit.symbol]]];
}

#pragma mark GUI controls
- (IBAction)setChangeTrigger:(id)sender {
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetVoltageInput_setVoltageChangeTrigger(ch, [changeTriggerSlider doubleValue]);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
    }
    [changeTriggerText setStringValue:[NSString stringWithFormat:@"%0.3f V", [changeTriggerSlider doubleValue]]];
    
}

- (IBAction)setDataInterval:(id)sender {
    PhidgetReturnCode result;
    NSEvent *currentEvent = [[sender window] currentEvent];
    if ([currentEvent type] == NSEventTypeLeftMouseUp) {
        result = PhidgetVoltageInput_setDataInterval(ch, [dataIntervalSlider intValue]);
        if(result != EPHIDGET_OK){
            [self outputLastError];
            return;
        }
    }
    [dataIntervalText setStringValue:[NSString stringWithFormat:@"%d ms", [dataIntervalSlider intValue]]];
    
}

- (IBAction)setVoltageRange:(id)sender {
    PhidgetReturnCode result;
    if([[voltageRangeCombo titleOfSelectedItem]  isEqual: @"10 mV"])
        result = PhidgetVoltageInput_setVoltageRange(ch, VOLTAGE_RANGE_10mV);
    else if([[voltageRangeCombo titleOfSelectedItem]  isEqual: @"40 mV"])
        result = PhidgetVoltageInput_setVoltageRange(ch, VOLTAGE_RANGE_40mV);
    else if([[voltageRangeCombo titleOfSelectedItem]  isEqual: @"200 mV"])
        result = PhidgetVoltageInput_setVoltageRange(ch, VOLTAGE_RANGE_200mV);
    else if([[voltageRangeCombo titleOfSelectedItem]  isEqual: @"312.5 mV"])
        result = PhidgetVoltageInput_setVoltageRange(ch, VOLTAGE_RANGE_312_5mV);
    else if([[voltageRangeCombo titleOfSelectedItem]  isEqual: @"400 mV"])
        result = PhidgetVoltageInput_setVoltageRange(ch, VOLTAGE_RANGE_400mV);
    else if([[voltageRangeCombo titleOfSelectedItem]  isEqual: @"1000 mV"])
        result = PhidgetVoltageInput_setVoltageRange(ch, VOLTAGE_RANGE_1000mV);
    else if([[voltageRangeCombo titleOfSelectedItem]  isEqual: @"2 V"])
        result = PhidgetVoltageInput_setVoltageRange(ch, VOLTAGE_RANGE_2V);
    else if([[voltageRangeCombo titleOfSelectedItem]  isEqual: @"5 V"])
        result = PhidgetVoltageInput_setVoltageRange(ch, VOLTAGE_RANGE_5V);
    else if([[voltageRangeCombo titleOfSelectedItem]  isEqual: @"15 V"])
        result = PhidgetVoltageInput_setVoltageRange(ch, VOLTAGE_RANGE_15V);
    else if([[voltageRangeCombo titleOfSelectedItem]  isEqual: @"40 V"])
        result = PhidgetVoltageInput_setVoltageRange(ch, VOLTAGE_RANGE_40V);
    else if([[voltageRangeCombo titleOfSelectedItem]  isEqual: @"Automatic"])
        result = PhidgetVoltageInput_setVoltageRange(ch, VOLTAGE_RANGE_AUTO);
	else
		result = EPHIDGET_UNEXPECTED;
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
}

- (IBAction)setSensorType:(id)sender {
    PhidgetReturnCode result;
    PhidgetVoltageInput_SensorType sensorType;
    result = PhidgetVoltageInput_setSensorType(ch, getSensorType([[sensorTypeCombo titleOfSelectedItem] cStringUsingEncoding:NSASCIIStringEncoding]));
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    result = PhidgetVoltageInput_getSensorType(ch, &sensorType);
    if(result != EPHIDGET_OK){
        [self outputLastError];
    }
    
    if(sensorType == SENSOR_TYPE_VOLTAGE){
        [self sensorModeChanged:NO];
    }
    else{
        [self sensorModeChanged:YES];
    }
}

#pragma mark Misc

-(void)sensorModeChanged:(BOOL)isSensorTypeEnabled{
    if(isSensorTypeEnabled){
        sensorText.textColor = [NSColor controlTextColor];
        voltageText.textColor = [NSColor disabledControlTextColor];
    }
    else{
        sensorText.textColor = [NSColor disabledControlTextColor];
        voltageText.textColor = [NSColor controlTextColor];
    }
}

@end
